/* ------------------------------------------------------------------------------------------------ */
/* FILE :		SysCall_ARMCC.c																		*/
/*																									*/
/* CONTENTS :																						*/
/*				System call layer extra for ARM CC													*/
/*				Provide the ARM CC library with all its native I/O interface						*/
/*																									*/
/*																									*/
/* Copyright (c) 2016-2019, Code-Time Technologies Inc. All rights reserved.						*/
/*																									*/
/* Code-Time Technologies retains all right, title, and interest in and to this work				*/
/*																									*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS							*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF										*/
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL							*/
/* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR								*/
/* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,							*/
/* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR							*/
/* OTHER DEALINGS IN THE SOFTWARE.																	*/
/*																									*/
/*																									*/
/*	$Revision: 1.7 $																				*/
/*	$Date: 2019/01/10 18:06:19 $																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#include "SysCall.h"

extern int G_UartDevOut;							/* The UART driver must know the device to use	*/

extern int  uart_send        (int Dev, const char *Buff, int Len);

const char __stdin_name[]  = ":stdin";				/* Variables holding the names used during the	*/
const char __stdout_name[] = ":stdout";				/* library initialization						*/
const char __stderr_name[] = ":stderr";

#ifdef __MICROLIB
  #error "SysCal_ARMCC is not compatible with microLib, use the standard library"
#endif

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    _sys_open																			*/
/*																									*/
/* _sys_open - open a file																			*/
/*																									*/
/* SYNOPSIS:																						*/
/*		FILEHANDLE _sys_open(const char *name, int openmode);										*/
/*																									*/
/* Refer to the compiler help or the file rt_sys.h for a description								*/
/* ------------------------------------------------------------------------------------------------ */

FILEHANDLE _sys_open(const char *name, int openmode)
{
FILEHANDLE fd;
int        flags;

	if (0 == strcmp(name, &__stdin_name[0])) {		/* Check for stdin / stdout / stderr			*/
		fd = (FILEHANDLE)0;							/* nothing special, report file descriptor #	*/
	}
	else if (0 == strcmp(name, &__stdout_name[0])) {
		fd = (FILEHANDLE)1;
	}
	else if (0 == strcmp(name, &__stderr_name[0])) {
		fd = (FILEHANDLE)2;
	}
	else {
		flags = -1;									/* To trap invalid openmode						*/

		if (openmode == OPEN_R) {					/* Open for reading								*/
			flags = O_RDONLY;
		}
		else if (openmode == OPEN_W) {				/* Truncate to 0 length or create for writing	*/
			flags = O_WRONLY
			      | O_CREAT
			      | O_TRUNC;
		}
		else if (openmode == OPEN_A) {				/* Append; open / create file for writing at EOF*/
			flags = O_WRONLY
			      | O_CREAT
			      | O_APPEND;
		}
		else if (openmode == (OPEN_R|OPEN_PLUS)) {	/* Open for update (reading and writing)		*/
			flags = O_RDWR;
		}
		else if (openmode == (OPEN_W|OPEN_PLUS)) {	/* Truncate to 0 length / create file for update*/
			flags = O_RDWR
			      | O_CREAT
			      | O_TRUNC;
		}
		else if (openmode == (OPEN_A|OPEN_PLUS)) {	/* Append; open or create file for update,		*/
			flags = O_RDWR							/* writing at end-of-file.						*/
			      | O_CREAT
			      | O_APPEND;
		}
		else {
			fd = (FILEHANDLE)-1;
		}

		if (flags != -1) {
			fd = (FILEHANDLE)open(name, flags, 0777);
		}
	}

	return(fd);
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    _sys_close																			*/
/*																									*/
/* _sys_open - close  a file																		*/
/*																									*/
/* SYNOPSIS:																						*/
/*		int _sys_close(FILEHANDLE fh);																*/
/*																									*/
/* Refer to the compiler help or the file rt_sys.h for a description								*/
/* ------------------------------------------------------------------------------------------------ */

int _sys_close(FILEHANDLE fh)
{
	return(close((int)fh));
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    _sys_write																			*/
/*																									*/
/* _sys_write - write to a file																		*/
/*																									*/
/* SYNOPSIS:																						*/
/*		int _sys_write(FILEHANDLE fh, const unsigned char *buf, unsigned len, int mode);			*/
/*																									*/
/* Refer to the compiler help or the file rt_sys.h for a description								*/
/* ------------------------------------------------------------------------------------------------ */

int _sys_write(FILEHANDLE fh, const unsigned char *buf, unsigned len, int mode)
{
int Ret;

	Ret = write((int)fh, buf, len);

	if (Ret >= 0) {
		Ret = len - Ret;							/* Returns # bytes not written					*/
	}												/* Of error (-ve), remain -ve					*/

	return(Ret);
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    _sys_read																			*/
/*																									*/
/* _sys_read - read from a file																		*/
/*																									*/
/* SYNOPSIS:																						*/
/*		int _sys_read(FILEHANDLE fh, unsigned char *buf, unsigned len, int mode);					*/
/*																									*/
/* Refer to the compiler help or the file rt_sys.h for a description								*/
/* ------------------------------------------------------------------------------------------------ */

int _sys_read(FILEHANDLE fh, unsigned char *buf, unsigned len, int mode)
{
int Ret;											/* Return value									*/

	Ret = read((int)fh, buf, len);

	if (Ret == 0) {
		Ret = 0x80000000
	        | len;
	}
	else if (Ret > 0) {
		Ret = len - Ret;							/* Returns # bytes not read						*/
	}

	return(Ret);
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    _ttywrch																			*/
/*																									*/
/* _ttywrch - basic output to stdout																*/
/*																									*/
/* SYNOPSIS:																						*/
/*		void _ttywrch(int ch);																		*/
/*																									*/
/* Refer to the compiler help or the file rt_sys.h for a description								*/
/* ------------------------------------------------------------------------------------------------ */

void _ttywrch(int ch)
{
char cch;

	cch = (char )ch;
	uart_send(G_UartDevOut, (const char *)&cch, 1);

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    _sys_isatty																			*/
/*																									*/
/* _sys_isatty - report if is a TTY																	*/
/*																									*/
/* SYNOPSIS:																						*/
/*		int _sys_isatty(FILEHANDLE fh);																*/
/*																									*/
/* Refer to the compiler help or the file rt_sys.h for a description								*/
/* ------------------------------------------------------------------------------------------------ */

int _sys_istty(FILEHANDLE fh)
{
	return(isatty(fh));
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    _sys_seek																			*/
/*																									*/
/* _sys_seek - re-adjust the r/w pointer in the file												*/
/*																									*/
/* SYNOPSIS:																						*/
/*		int _sys_seek(FILEHANDLE fh, long pos);														*/
/*																									*/
/* Refer to the compiler help or the file rt_sys.h for a description								*/
/* ------------------------------------------------------------------------------------------------ */

int _sys_seek(FILEHANDLE fh, long pos)
{
	return(lseek(fh, pos, SEEK_SET));
}
#if 0
/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    _sys_ensure																			*/
/*																									*/
/* _sys_ensure - flush a file (from internal buffer to device)										*/
/*																									*/
/* SYNOPSIS:																						*/
/*		int _sys_ensure(FILEHANDLE fh);																*/
/*																									*/
/* Refer to the compiler help or the file rt_sys.h for a description								*/
/* ------------------------------------------------------------------------------------------------ */

int _sys_ensure(FILEHANDLE fh)
{
	return(-1);
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    _sys_flen																			*/
/*																									*/
/* _sys_flen - report the size of a file															*/
/*																									*/
/* SYNOPSIS:																						*/
/*		long _sys_flen(FILEHANDLE fh);																*/
/*																									*/
/* Refer to the compiler help or the file rt_sys.h for a description								*/
/* ------------------------------------------------------------------------------------------------ */

long _sys_flen(FILEHANDLE fh)
{
	return((long)-1);
}
#endif
/* EOF */
